Class: Actions::Katello::Repository::Destroy

Inherits:
EntryAction
  • Object
show all
Defined in:
app/lib/actions/katello/repository/destroy.rb

Instance Method Summary collapse

Instance Method Details

#check_destroyable!(repository, remove_from_content_view_versions) ⇒ Object



131
132
133
134
135
136
137
# File 'app/lib/actions/katello/repository/destroy.rb', line 131

def check_destroyable!(repository, remove_from_content_view_versions)
  unless repository.destroyable?(remove_from_content_view_versions)
    # The repository is going to be deleted in finalize, but it cannot be deleted.
    # Stop now and inform the user.
    fail repository.errors.messages.values.join("\n")
  end
end

#delete_empty_repo_filters(repository) ⇒ Object



81
82
83
84
# File 'app/lib/actions/katello/repository/destroy.rb', line 81

def delete_empty_repo_filters(repository)
  filters_to_delete = repository.filters.select { |filter| filter.repositories.size == 1 }
  ::Katello::ContentViewFilter.where(id: filters_to_delete).destroy_all
end

#delete_record(repository, options = {}) ⇒ Object



103
104
105
106
107
108
# File 'app/lib/actions/katello/repository/destroy.rb', line 103

def delete_record(repository, options = {})
  ::Katello::SyncPlan.remove_disabled_product(repository) if repository.redhat?
  repository.destroy!
  repository.root.destroy! if repository.root.repositories.empty?
  ::Katello::DockerMetaTag.cleanup_tags if options[:docker_cleanup]
end

#finalizeObject



46
47
48
49
50
51
52
53
54
55
56
57
58
59
# File 'app/lib/actions/katello/repository/destroy.rb', line 46

def finalize
  repository = ::Katello::Repository.find_by(id: input[:repository][:id])
  if repository
    docker_cleanup = repository.docker? && input[:docker_cleanup]
    delete_record(repository, {docker_cleanup: docker_cleanup})

    if (affected_cvv_ids = input[:affected_cvv_ids]).any?
      cvvs = ::Katello::ContentViewVersion.where(id: affected_cvv_ids)
      cvvs.each do |cvv|
        cvv.update_content_counts!
      end
    end
  end
end

#handle_acs_product_removal(repository) ⇒ Object



67
68
69
70
71
72
73
74
75
76
77
78
79
# File 'app/lib/actions/katello/repository/destroy.rb', line 67

def handle_acs_product_removal(repository)
  # Remove products from ACS's that contain no repositories which both
  # match the ACS content type and have a non-nil URL
  product = repository.product
  repo_content_types = ::Katello::RootRepository.where(:id => product.repositories.where.not(:id => repository.id).select(:root_id)).where.not(:url => [nil, '']).distinct.pluck(:content_type)
  ::Katello::AlternateContentSource.with_products(product).each do |acs|
    unless repo_content_types.include?(acs.content_type)
      acs.products = acs.products - [product]
      Rails.logger.info _('Removing product %{prod_name} with ID %{prod_id} from ACS %{acs_name} with ID %{acs_id}') %
        { prod_name: product.name, prod_id: product.id, acs_name: acs.name, acs_id: acs.id }
    end
  end
end

#handle_alternate_content_sources(repository) ⇒ Object



61
62
63
64
65
# File 'app/lib/actions/katello/repository/destroy.rb', line 61

def handle_alternate_content_sources(repository)
  repository.smart_proxy_alternate_content_sources.each do |smart_proxy_acs|
    plan_action(Pulp3::Orchestration::AlternateContentSource::Delete, smart_proxy_acs)
  end
end

#handle_custom_content(repository, remove_from_content_view_versions) ⇒ Object



86
87
88
89
90
91
92
93
94
# File 'app/lib/actions/katello/repository/destroy.rb', line 86

def handle_custom_content(repository, remove_from_content_view_versions)
  # If this is the last instance of a custom repo or a deb repo, then destroy the content
  if remove_from_content_view_versions || repository.root.repositories.where.not(id: repository.id).empty? || repository.deb?
    # Never destroy content for deb rolling repo clones, it belongs to the library instance!
    return if repository.deb? && repository.content_view.rolling?

    plan_action(::Actions::Katello::Product::ContentDestroy, repository)
  end
end

#handle_redhat_content(repository) ⇒ Object



96
97
98
99
100
101
# File 'app/lib/actions/katello/repository/destroy.rb', line 96

def handle_redhat_content(repository)
  content_view_environment = repository.content_view.content_view_environment(repository.environment)
  if content_view_environment && !repository.content_view.rolling?
    plan_action(Candlepin::Environment::SetContent, repository.content_view, repository.environment, content_view_environment)
  end
end

#humanized_nameObject



139
140
141
# File 'app/lib/actions/katello/repository/destroy.rb', line 139

def humanized_name
  _("Delete")
end

#plan(repository, options = {}) ⇒ Object

options:

skip_environment_update - defaults to false. skips updating the CP environment
destroy_content - can be disabled to skip Candlepin remove_content


10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
# File 'app/lib/actions/katello/repository/destroy.rb', line 10

def plan(repository, options = {})
  affected_cvv_ids = []
  skip_environment_update = options.fetch(:skip_environment_update, false) ||
      options.fetch(:organization_destroy, false)
  destroy_content = options.fetch(:destroy_content, true)
  remove_from_content_view_versions = options.fetch(:remove_from_content_view_versions, false)
  delete_empty_repo_filters = options.fetch(:delete_empty_repo_filters, true)
  docker_cleanup = options.fetch(:docker_cleanup, true)
  action_subject(repository)
  check_destroyable!(repository, remove_from_content_view_versions)
  remove_generated_content_views(repository)

  remove_versions(repository, repository.content_views.generated_for_library, affected_cvv_ids)

  plan_action(Actions::Pulp3::Orchestration::Repository::Delete,
                   repository,
                   SmartProxy.pulp_primary)

  remove_versions(repository, repository.content_views_all(include_composite: true)&.generated_for_none, affected_cvv_ids) if remove_from_content_view_versions

  handle_acs_product_removal(repository)
  handle_alternate_content_sources(repository)
  delete_empty_repo_filters(repository) if delete_empty_repo_filters

  plan_self(:user_id => ::User.current.id, :affected_cvv_ids => affected_cvv_ids, :docker_cleanup => docker_cleanup)
  sequence do
    if repository.redhat?
      handle_redhat_content(repository) unless skip_environment_update
    else
      if destroy_content && !skip_environment_update
        handle_custom_content(repository, remove_from_content_view_versions)
      end
    end
  end
end

#remove_generated_content_views(repository) ⇒ Object



110
111
112
113
114
115
116
117
118
# File 'app/lib/actions/katello/repository/destroy.rb', line 110

def remove_generated_content_views(repository)
  # remove the content views generated for this repository (since we are deleting the repo)
  content_views = repository.content_views.generated_for_repository
  return if content_views.blank?
  plan_action(::Actions::BulkAction, ::Actions::Katello::ContentView::Remove,
                content_views,
                skip_repo_destroy: true,
                destroy_content_view: true)
end

#remove_versions(repository, content_views, affected_cvv_ids) ⇒ Object



120
121
122
123
124
125
126
127
128
129
# File 'app/lib/actions/katello/repository/destroy.rb', line 120

def remove_versions(repository, content_views, affected_cvv_ids)
  return if content_views.blank?
  interested_inverses = repository.
                          library_instances_inverse.
                          joins(:content_view_version => :content_view).
                          merge(content_views)
  return if interested_inverses.blank?
  affected_cvv_ids.concat(interested_inverses.pluck(:content_view_version_id)&.uniq)
  plan_action(::Actions::BulkAction, ::Actions::Katello::Repository::Destroy, interested_inverses)
end