Module: PulpRpmClient

Defined in:
lib/monkeys/remove_overwrite_parameter.rb

Overview

Monkey patch to remove the ‘overwrite’ parameter from RepositoryAddRemoveContent

The pulp_rpm_client gem 3.35.3 includes an ‘overwrite’ parameter that was added in a newer Pulp RPM version (3.37.0+), but the current Pulp server doesn’t support it yet. This patch removes the parameter from the serialized hash to maintain compatibility.

Defined Under Namespace

Classes: RepositoryAddRemoveContent